@@ -2,13 +2,122 @@ package Audio::Nama::Graph;
 use Modern::Perl;
 use Carp;
 use Graph;
-use vars qw(%reserved $debug $debug2);
+use Audio::Nama::Util qw(input_node output_node);
+use Log::Log4perl;
+use Audio::Nama::Log qw(logsub);
+use vars qw(%reserved);
 # this dispatch table also identifies labels reserved
 # for signal sources and sinks.
 *reserved = \%Audio::Nama::IO::io_class;
-*debug = \$Audio::Nama::debug;
-*debug2 = \$Audio::Nama::debug2;
 
+our $logger;
+
+sub initialize_logger { $logger = Log::Log4perl->get_logger() }
+
+sub add_path_for_rec {
+
+	# connect input source to file 
+	
+	my($g,$track) = @_;
+
+	logsub("&add_path_for_rec: track ".$track->name);
+	# Track input from a WAV, JACK client, or soundcard
+	# Record 'raw' signal
+	#
+	# Do *not* record signals if the source reports it is
+	# a track, bus or loop
+
+	if( $track->source_type !~ /track|bus|loop/ )
+	{
+		# create temporary track for rec_file chain
+
+		# (it may be possible to avoid creating a 
+		# temporary track by providing the data as 
+		# graph edge attributes)
+
+		$logger->debug("rec file link for ".$track->name);
+		my $name = $track->name . '_rec_file';
+		my $anon = Audio::Nama::SlaveTrack->new( 
+			target => $track->name,
+			rw => 'OFF',
+			group => 'Temp',
+			name => $name);
+
+		# connect writepath: source --> temptrackname --> wav_out
+		
+		$g->add_path(input_node($track->source_type), $name, 'wav_out');
+
+
+		$g->set_vertex_attributes($name, { 
+
+			# set chain_id to R3 (if original track is 3) 
+
+			chain_id => 'R'.$track->n,
+
+			# do not perform mono-to-stereo copy,
+			# (override IO class default)
+
+			mono_to_stereo => '', 
+		});
+
+	} 
+	elsif ($track->source_type =~ /bus|track/) 
+	{
+
+		# for tracks with identified (track|bus) input
+
+		# cache_tracks/merge_edits has its own logic
+		# therefore these connections (triggered from
+		# generate_setup()) will not affect AFAIK
+		# any other recording scenario
+
+		# special case, record 'cooked' signal
+
+		# generally a sub bus 
+
+		# - has 'rec_defeat' set (therefore doesn't reach here)
+		# - receives a stereo input
+		# - mix track width is set to stereo (default)
+
+		my @edge = ($track->name, 'wav_out'); # cooked signal
+
+		$g->add_path(@edge); 
+
+		# set chain_id to R3 (if original track is 3) 
+
+		$g->set_edge_attributes(@edge, { 
+			chain_id => 'R'.$track->n,
+		});
+		
+		# if this path is left unconnected, 
+		# i.e. track gets no input		
+		# it will be removed by prune_graph()
+		
+		# to record raw:
+		
+		# source_type: loop
+		# source_id:   loop,track_name_in
+
+		# but for WAV to contain content, 
+		# we need to guarantee that track_name has
+		# an input
+	}
+}
+sub add_path_for_aux_send {
+	my ($g, $track) = @_;
+		logsub("&add_path_for_aux_send: track ".$track->name);
+		# for track 'sax', send_type 'jack_client', create route as 
+		# sax-jack_client_out
+		my @edge = ($track->name, output_node($track->send_type));
+		$g->add_edge(@edge);
+		 $g->set_edge_attributes(
+				@edge,
+			  	{	track => $track->name,
+					width => 2, # force stereo output width
+					chain_id => 'S'.$track->n,
+				});
+}
+{
 my %seen;
 
 sub expand_graph {
@@ -19,37 +128,31 @@ sub expand_graph {
 	
 	for ($g->edges){
 		my($a,$b) = @{$_}; 
-		$debug and say "$a-$b: processing...";
-		$debug and say "$a-$b: already seen" if $seen{"$a-$b"};
+		$logger->debug("$a-$b: processing...");
+		$logger->debug("$a-$b: already seen") if $seen{"$a-$b"};
 		next if $seen{"$a-$b"};
 
 		# case 1: both nodes are tracks: default insertion logic
 	
 		if ( is_a_track($a) and is_a_track($b) ){ 
-			$debug and say "processing track-track edge: $a-$b";
+			$logger->debug("processing track-track edge: $a-$b");
 			add_loop($g,$a,$b) } 
 
 		# case 2: fan out from track: use near side loop
 
 		elsif ( is_a_track($a) and $g->successors($a) > 1 ) {
-			$debug and say "fan_out from track $a";
+			$logger->debug("fan_out from track $a");
 			add_near_side_loop($g,$a,$b,out_loop($a));}
 	
 		# case 3: fan in to track: use far side loop
 		
 		elsif ( is_a_track($b) and $g->predecessors($b) > 1 ) {
-			$debug and say "fan in to track $b";
+			$logger->debug("fan in to track $b");
 			add_far_side_loop($g,$a,$b,in_loop($b));}
-		else { $debug and say "$a-$b: no action taken" }
+		else { $logger->debug("$a-$b: no action taken") }
 	}
 	
 }
-sub add_path {
-	my @nodes = @_;
-	$debug and say "adding path: ", join " ", @nodes;
-	$Audio::Nama::g->add_path(@nodes);
-}
-sub add_edge { add_path(@_) }
 	
 sub add_inserts {
 	my $g = shift;
@@ -64,11 +167,11 @@ sub add_inserts {
 
 sub add_loop {
 	my ($g,$a,$b) = @_;
-	$debug and say "adding loop";
+	$logger->debug("adding loop");
 	my $fan_out = $g->successors($a);
-	$debug and say "$a: fan_out $fan_out";
+	$logger->debug("$a: fan_out $fan_out");
 	my $fan_in  = $g->predecessors($b);
-	$debug and say "$b: fan_in $fan_in";
+	$logger->debug("$b: fan_in $fan_in");
 	if ($fan_out > 1){
 		add_near_side_loop($g,$a,$b, out_loop($a))
 	} elsif ($fan_in  > 1){
@@ -129,7 +232,7 @@ sub add_loop {
 # the track number and will therefore get the track effects
 
  	my ($g, $a, $b, $loop) = @_;
- 	$debug and say "$a-$b: insert near side loop";
+ 	$logger->debug("$a-$b: insert near side loop");
 	# we will insert loop _after_ processing successor
 	# edges so $a-$loop will not be picked up 
 	# in successors list.
@@ -148,45 +251,39 @@ sub add_loop {
 		track => $Audio::Nama::tn{$a}->name});
 	map{ 
  		my $attr = $g->get_edge_attributes($a,$_);
- 		$debug and say "deleting edge: $a-$_";
+ 		$logger->debug("deleting edge: $a-$_");
  		$g->delete_edge($a,$_);
- 		$debug and say "adding edge: $loop-$_";
-		add_edge($loop, $_);
+		$g->add_edge($loop, $_);
 		$g->set_edge_attributes($loop,$_, $attr) if $attr;
 		$seen{"$a-$_"}++;
  	} $g->successors($a);
-	$debug and say "adding edge: $a-$loop";
-	add_edge($a,$loop);
+	$g->add_edge($a,$loop);
 }
  
 
 sub add_far_side_loop {
  	my ($g, $a, $b, $loop) = @_;
- 	$debug and say "$a-$b: insert far side loop";
+ 	$logger->debug("$a-$b: insert far side loop");
 	
 	$g->set_vertex_attributes($loop,{
 		n => $Audio::Nama::tn{$a}->n, j => 'a',
 		track => $Audio::Nama::tn{$a}->name});
 	map{ 
  		my $attr = $g->get_edge_attributes($_,$b);
- 		$debug and say "deleting edge: $_-$b";
+ 		$logger->debug("deleting edge: $_-$b");
  		$g->delete_edge($_,$b);
- 		$debug and say "adding edge: $_-$loop";
-		add_edge($_,$loop);
+		$g->add_edge($_,$loop);
 		$g->set_edge_attributes($_,$loop, $attr) if $attr;
 		$seen{"$_-$b"}++;
  	} $g->predecessors($b);
-	$debug and say "adding edge: $loop-$b";
-	add_edge($loop,$b);
+	$g->add_edge($loop,$b);
 }
 
+}
 
 sub in_loop{ "$_[0]_in" }
 sub out_loop{ "$_[0]_out" }
-#sub is_a_track{ $Audio::Nama::tn{$_[0]} }
-sub is_a_track{ return unless $_[0] !~ /_(in|out)$/;}
-# $debug and say "$_[0] is a track"; 1
-#}
+sub is_a_track{ $Audio::Nama::tn{$_[0]} }  # most reliable
 sub is_terminal { $reserved{$_[0]} }
 sub is_a_loop{
 	my $name = shift;
@@ -204,26 +301,49 @@ sub inputless_tracks {
 	my $g = shift;
 	(grep{ is_a_track($_) and $g->is_source_vertex($_) } $g->vertices)
 }	
-sub remove_inputless_tracks {
+sub remove_out_of_bounds_tracks {
 	my $g = shift;
-	while(my @i = Audio::Nama::Graph::inputless_tracks($g)){
-		map{ 	$g->delete_edges(map{@$_} $g->edges_from($_));
-				$g->delete_vertex($_);
-		} @i;
+	my @names = $g->successors('wav_in');  # MON status tracks
+	map{ remove_tracks($g, $_) } 
+	grep{
+		Audio::Nama::set_edit_vars($Audio::Nama::tn{$_});
+		Audio::Nama::edit_case() =~ /out_of_bounds/
+	} @names;
+}
+
+sub recursively_remove_inputless_tracks {
+	my $g = shift;
+	# make multiple passes if necessary
+	while(my @i = inputless_tracks($g)){
+		remove_tracks($g, @i);
 	}
 }
 sub outputless_tracks {
 	my $g = shift;
 	(grep{ is_a_track($_) and $g->is_sink_vertex($_) } $g->vertices)
 }	
-sub remove_outputless_tracks {
+sub recursively_remove_outputless_tracks {
 	my $g = shift;
-	while(my @i = Audio::Nama::Graph::outputless_tracks($g)){
-		map{ 	$g->delete_edges(map{@$_} $g->edges_to($_));
-				$g->delete_vertex($_);
-		} @i;
+	while(my @i = outputless_tracks($g)){
+		remove_tracks($g, @i);
 	}
 }
+sub remove_tracks {
+	my ($g, @names) = @_;
+		map{ 	$g->delete_edges(map{@$_} $g->edges_from($_));
+				$g->delete_edges(map{@$_} $g->edges_to($_));
+				$g->delete_vertex($_);
+		} @names;
+}
+
+### we need jack clients latency 
+sub add_jack_io {
+	my $g = shift;
+		
+
+}
+	
+		
 		
 1;
 __END__
@@ -239,7 +359,7 @@ If we are to record the input, we need:
 
 	sax -> wav_out
 
-If we add an instrument monitor for the sax player, we need:
+If we add an instrument monitor on a separate channel for the sax player, we need:
 
 	sax -> soundcard_out
 
@@ -248,14 +368,13 @@ must fan out or fan in.
 
 	soundcard_in -> sax -> sax_out -> Master -> soundcard_out
 
-	sax_out -> wav_out
+	                       sax_out -> wav_out
 
-	sax_out -> soundcard_out
+	                       sax_out -> soundcard_out
 
 Here 'sax_out' is a loop device.
 
-Though there are more complicated additions, such as inserts,
-they must follow these same rules.
+All routing functions follow these rules.
 
 We then process each edge to generate a line for the Ecasound chain setup
 file.
@@ -264,10 +383,11 @@ Master -> soundcard_out is easy to process, because the track
 Master knows what it's outputs should be.
 
 The edge sax_out -> soundcard_out, an auxiliary send, needs to know its
-associated track, as well as the chain_id, the identifier for the Ecasound
-chain corresponding to this edge.
+associated track, the chain_id (identifier for the Ecasound
+chain corresponding to this edge) and in the final step
+the soundcard channel number.
 
-We provide this information as edge attributes.
+We can provide this information as edge attributes.
 
 We also allow vertexes, for example a track or loop device, to carry data is
 well, for example to tell the dispatcher to override the 
