@@ -3,6 +3,51 @@ use Modern::Perl;
 use Carp;
 use Audio::Nama::Assign qw(yaml_out); 
 
+=comment
+{
+	package Audio::Nama::Does::Serialize;
+	use Role::Basic;
+	requires 'as_hash';
+	sub as_hash {
+		my $self = shift;
+		my $class = ref $self;
+		bless $self, 'HASH'; # easy magic
+		#print yaml_out $self; return;
+		my %guts = %{ $self };
+		#print join " ", %guts; return;
+		#my @keys = keys %guts;
+		#map{ $output->{$_} or $output->{$_} = '~'   } @keys; 
+		bless $self, $class; # restore
+		return \%guts; # *not* a copy, a risk, but we
+                       # are serializing, not altering 
+	}
+}
+
+Audio::Nama::Does::Persist
+	# later,
+	# for the class, provides an
+	# (possibly filtered, altered) 
+	# array of objects.
+
+
+
+  In a role:
+
+1;
+
+  In your class:
+           
+           package My::Class;
+           use Role::Basic 'with';
+           
+           with qw(
+               Does::Serialize::AsYAML
+           );
+           
+           sub as_hash { ... } # because the role requires it
+
+=cut
+
 no strict; # Enable during dev and testing
 BEGIN {
 	require 5.004;
@@ -12,16 +57,16 @@ BEGIN {
 sub import {
 	return unless shift eq 'Audio::Nama::Object';
 	my $pkg   = caller;
-	my $child = !! @{"${pkg}::ISA"};
+	my $child = 0+@{"${pkg}::ISA"};
 	eval join '',
 		"package $pkg;\n",
 		' use vars qw(%_is_field);   ',
 		' map{ $_is_field{$_}++ } @_;',
-		($child ? () : "\@${pkg}::ISA = 'Audio::Nama::Object';\n"),
+		($child ? () : "\@${pkg}::ISA = Audio::Nama::Object;\n"),
 		map {
 			defined and ! ref and /^[^\W\d]\w*$/s
 			or die "Invalid accessor name '$_'";
-			"sub $_ { return \$_[0]->{$_} }\n"
+			"sub $_ : lvalue { \$_[0]->{$_} }"
 		} @_;
 	die "Failed to generate $pkg" if $@;
 	return 1;
@@ -33,11 +78,25 @@ sub new {
 }
 
 sub is_legal_key { 
+
+	# The behavior I want here is:
+	#
+	# Example class hierachy: Audio::Nama::Object, Audio::Nama::Wav, Audio::Nama::Track, Audio::Nama::SimpleTrack
+	
+	# By inheriting from Track, SimpleTrack gets all the
+	# attributes of Track and Wav, without having to include
+	# them in the Track class definition
+	
 	my ($class, $key) = @_;
 	$class = ref $class if ref $class;  # support objects
 	return 1 if ${"$class\::_is_field"}{$key};
 	my ($parent_class) = @{"$class\::ISA"};
+
 	return unless $parent_class and $parent_class !~ /Object::Tiny/;
+
+	# this should be:
+	# return unless $parent_class and $parent_class !~ /Object/;
+	
 	is_legal_key($parent_class,$key);
 }
 sub set {
@@ -52,14 +111,6 @@ sub set {
 			is_legal_key(ref $self, $key) or croak "illegal key: $_ for object of type ", ref $self;
 	} keys %new_vals;
 }
-# sub ancestors {
-# 	my $class = ref $_[0];
-# 	$class, parents( @{"$class\::ISA"} );
-# }
-# 
-# sub parents {
-# 	my @ISA = @_;
-# 	map{ 
 sub dumpp  {
 	my $self = shift;
 	my $class = ref $self;
@@ -77,7 +128,7 @@ sub dump {
 	bless $self, $class; # restore
 	return $output;
 }
-sub hashref {
+sub as_hash {
 	my $self = shift;
 	my $class = ref $self;
 	bless $self, 'HASH'; # easy magic
@@ -98,21 +149,32 @@ __END__
 
 =head1 NAME
 
-::Object - Class builder
+Audio::Nama::Object - Class builder
 
 =head1 SYNOPSIS
 
   # Define a class
   package Foo;
   
-  use Audio::Nama::Object qw{ bar baz };
+  use Audio::Nama::Object qw{ bux baz };
   
   1;
   
   
   # Use the class
-  my $object = Foo->new( bar => 1 );
+  my $object = Foo->new( bux => 1 );
 
-  $object->set( bar => 2);
+  $object->set( bux => 2);
   
-  print "bar is " . $object->bar . "\n";
\ No newline at end of file
+  print "bux is " . $object->bux . "\n";
+
+
+  # Define a subclass (automatically inherits parent attributes)
+
+  package Bar;
+
+  our @ISA = 'Foo';
+
+  my $lonely_bar = Bar->new();
+  
+  $lonely_bar->set(bux => 3); 
\ No newline at end of file
