@@ -1,24 +1,25 @@
 package Audio::Nama::Wav;
 our $VERSION = 1.0;
 our @ISA; 
-use Audio::Nama::Object qw(name active dir);
+use Audio::Nama::Object qw(name version dir);
 use warnings;
 use Audio::Nama::Assign qw(:all);
-use Memoize qw(memoize unmemoize);
+use Audio::Nama::Util qw(join_path);
+use Audio::Nama::Log qw(logit);
+use Memoize qw(memoize unmemoize); # called by code in Audio::Nama::Memoize.pm
 no warnings qw(uninitialized);
 use Carp;
 
 sub get_versions {
-	#local $debug = 1;
 	my $self = shift;
 	my ($sep, $ext) = qw( _ wav );
 	my ($dir, $basename) = ($self->dir, $self->basename);
 #	print "dir: ", $self->dir(), $/;
 	#print "basename: ", $self->basename(), $/;
-	$debug and print "getver: dir $dir basename $basename sep $sep ext $ext\n\n";
+	logit(__LINE__,__PACKAGE__,'debug',"getver: dir $dir basename $basename sep $sep ext $ext");
 	my %versions = ();
 	for my $candidate ( candidates($dir) ) {
-	#	$debug and print "candidate: $candidate\n\n";
+	#	logit(__LINE__,__PACKAGE__,'debug',"candidate: $candidate");
 	
 		my( $match, $dummy, $num) = 
 			( $candidate =~ m/^ ( $basename 
@@ -28,7 +29,7 @@ sub get_versions {
 			  ); # regex statement
 		if ( $match ) { $versions{ $num || 'bare' } =  $match }
 	}
-	$debug and print "get_version: " , Audio::Nama::yaml_out(\%versions);
+	logit(__LINE__,__PACKAGE__,'debug',sub{"get_version: " , Audio::Nama::yaml_out(\%versions)});
 	%versions;
 }
 
@@ -39,7 +40,7 @@ sub candidates {
 	my @candidates = readdir $wavdir;
 	closedir $wavdir;
 	@candidates = grep{ ! (-s join_path($dir, $_) == 44 ) } @candidates;
-	#$debug and print join $/, @candidates;
+	#$logit(__LINE__,__PACKAGE__,'debug',join $/, @candidates);
 	@candidates;
 }
 
@@ -53,7 +54,7 @@ sub targets {
 		if ($versions{bare}) {  $versions{1} = $versions{bare}; 
 			delete $versions{bare};
 		}
-	$debug and print "\%versions\n================\n", yaml_out(\%versions);
+	logit(__LINE__,__PACKAGE__,'debug',sub{"\%versions\n================\n", yaml_out(\%versions)});
 	\%versions;
 }
 
@@ -66,4 +67,6 @@ sub versions {
 
 sub last { 
 	my $self = shift;
-	pop @{ $self->versions} }
\ No newline at end of file
+	pop @{ $self->versions} }
+
+1;
\ No newline at end of file
